﻿// <summary>ソースコード：ダム（一定量、定率定量方式）モデルクラス</summary>
// <author>CommonMP</author>

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreImpl.Tool;

// ToDo namespace は　モデル開発者が変更して、ユニークな名称にして下さい
namespace jp.or.jccajsce.DamConstantRateDischargingModel
{
    /// <summary><para>class outline:</para>
    /// <para>ダム（一定量、定率定量方式）モデルクラス</para>
    /// </summary>
    /// <remarks><para>history:</para>
    /// <para>[CommonMP][ver 1.0.0][2014/08][新規作成]</para>
    
    public class DamConstantRateDischargingModel : McStateCalModelBase
    {
        /// <summary> 演算データ（キャスト用） </summary>
        DamConstantRateDischargingCalInfo m_csRMInf = null;   // 便利の為、キャスト用に定義しておく

        //=========================
        // 演算実行前処理関連メソッド
        //=========================

        /// <summary><para>method outline:</para>
        /// <para>入力側の接続情報チェック</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>bool bRtn = ReceiveConnectionCheck(ref csErrorInf, lInputDataNum, csInputCellData)</para>
        /// </example>
        /// <param name="csErrorInf">エラー出力</param>
        /// <param name="lInputDataNum">入力情報数</param>
        /// <param name="csInputCellData">演算に必要な入力情報配列</param>
        /// <returns>=true:正常、=false:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>受信するデータが自モデルが期待している情報か否かをチェックする</para>
        /// </remarks>
        protected override bool ReceiveConnectionCheck(ref McStructErrorInfo csErrorInf, long lInputDataNum, McReceiveCellDataIF[] csInputCellData)
        {
            bool bRtn = true;

            if (lInputDataNum == 0)
            {
                csErrorInf.AddCheckErrorData(this.GetID(), DamConstantRateDischargingDefine.DamConstantRateDischarging_MODEL_KIND,
                    "No receive data");
                return false;
            }

            for (long IP = 0; IP < lInputDataNum; IP++)
            {
                // 入力データ数分
                if (DamConstantRateDischargingDefine.IN_PATTERN.Equals(csInputCellData[IP].GetReceivePatternID()) == true
                    || DamConstantRateDischargingDefine.IN_PATTERN_P.Equals(csInputCellData[IP].GetReceivePatternID())) 
                {
                    // 20140926
                    // 要求されるセル数は常に１。より小さいことは無いのでエラー判定不要。
                    // 各セル内で要求されるデータ数も１個。より小さいことは無いのでエラー判定不要
                    // 受信自体は１種類なので個別に受信接続数やその配列での位置を管理する必要はない
                }
                else
                {
                    csErrorInf.AddCheckErrorData(this.GetID(), DamConstantRateDischargingDefine.DamConstantRateDischarging_MODEL_KIND,
                        "Unexpected receive data type. (Received from " + csInputCellData[IP].GetUpperElementID().ToString() + ")");
                    bRtn = false;
                }
            }

            return bRtn;
        }

        /// <summary><para>method outline:</para>
        /// <para>出力側の接続情報チェック</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>bool bRtn = SendConnectionCheck(ref csErrorInf, lOutputDataNum, csOutputCellData)</para>
        /// </example>
        /// <param name="csErrorInf">エラー出力</param>
        /// <param name="lOutputDataNum">出力情報数</param>
        /// <param name="csOutputCellData">出力情報配列</param>
        /// <returns>=true:正常、=false:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>送信端子に設定されている伝送データが自モデルが期待している情報か否かをチェックする</para>
        /// </remarks>
        /// 

        protected override bool SendConnectionCheck(ref McStructErrorInfo csErrorInf, long lOutputDataNum, McSendCellDataIF[] csOutputCellData)
         {
            bool bRtn = true;


            // 20140926
            // 接続が３種類あるためDataFusionでの負荷を減らすために接続数と配列での位置データを作成した方がよい
            List<long> listFlowIdx = new List<long>();
            List<long> listStageIdx = new List<long>();
            List<long> listStorageIdx = new List<long>();

            for (long lLp = 0; lLp < lOutputDataNum; lLp++)
            {   // 出力する伝送データ数分繰り返します。
                // 20140926
                // 各伝送内容の例外判定は入力接続と同様の理由で不要

                if (DamConstantRateDischargingDefine.OUT_FLOW_PATTERN.Equals(csOutputCellData[lLp].GetSendPatternID()) == true
                    || DamConstantRateDischargingDefine.OUT_FLOW_PATTERN_P.Equals(csOutputCellData[lLp].GetSendPatternID()))
                {
                    listFlowIdx.Add(lLp);
                }
                else if (DamConstantRateDischargingDefine.OUT_STAGE_PATTERN.Equals(csOutputCellData[lLp].GetSendPatternID()) == true
                    || DamConstantRateDischargingDefine.OUT_STAGE_PATTERN_P.Equals(csOutputCellData[lLp].GetSendPatternID()))
                {
                    listStageIdx.Add(lLp);
                }
                else if (DamConstantRateDischargingDefine.OUT_STORAGE_PATTERN.Equals(csOutputCellData[lLp].GetSendPatternID()) == true
                    || DamConstantRateDischargingDefine.OUT_STORAGE_PATTERN_P.Equals(csOutputCellData[lLp].GetSendPatternID()))
                {
                    listStorageIdx.Add(lLp);
                }
                else
                {
                    csErrorInf.AddCheckErrorData(this.GetID(), DamConstantRateDischargingDefine.DamConstantRateDischarging_MODEL_KIND,
                        "Unexpected send data type");
                    bRtn = false;
                }
            }

            if (bRtn)
            {
                this.m_csRMInf.m_lOutDataQNum = listFlowIdx.Count;
                this.m_csRMInf.m_lOutQIndx = listFlowIdx.ToArray();

                this.m_csRMInf.m_lOutDataHNum = listStageIdx.Count;
                this.m_csRMInf.m_lOutHIndx = listStageIdx.ToArray();

                this.m_csRMInf.m_lOutDataVNum = listStorageIdx.Count;
                this.m_csRMInf.m_lOutVIndx = listStorageIdx.ToArray();
            }
            return bRtn;
        }
        
        /// <summary><para>method outline:</para>
        /// <para>モデルを初期化する</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>Initialize(csInitialData, lInputDataNum, csInputCellData)</para>
        /// </example>
        /// <param name="csInitialData">初期化設定情報</param>
        /// <param name="lInputDataNum">入力情報数</param>
        /// <param name="csInputCellData">演算に必要な入力情報配列</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        protected override bool Initialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
        {
            bool bRtn = false;
            // 引数で与えられたデータを　キャストしています。
            McInitialInfo csInDt = csInitialData as McInitialInfo;
            if (csInDt != null)
            {
                csInDt.GetInfo("m_dInitStage", ref m_csRMInf.m_dInit);

                bRtn = true;
            }

            m_csRMInf.m_dQout = 0.0;  //放流量
            m_csRMInf.m_dHdam = m_csRMInf.m_dInit;  //貯水位
            m_csRMInf.m_dStorage = 0.0;  //貯水量
            m_csRMInf.m_dQin = 0.0;  //流入量

            return bRtn;
        }
        /// <summary><para>method outline:</para>
        /// <para>モデルを初期化する</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>OnlineInitialize(csInitialData)</para>
        /// </example>
        /// <param name="csInitialData">初期化設定情報</param>
        /// <param name="lInputDataNum">入力情報数</param>
        /// <param name="csInputCellData">演算に必要な入力情報配列</param>
        /// <returns>bool</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        protected override bool OnlineInitialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
        {
            // 
            return true;
        }
        //=======================
        // 演算実行処理関連メソッド
        //=======================

        /// <summary><para>method outline:</para>
        /// <para>モデル演算</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>long lRtn = Calculate(ref csInputCellData)</para>
        /// </example>
        /// <param name="lInputDataNum">入力情報数</param>
        /// <param name="csInputCellData">演算に必要な入力情報配列</param>
        /// <returns>=0:正常 -1:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        protected override long Calculate(long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
        {
            double dDt = m_csDltTime.GetTime();
            HySCellData[] csCellDt = null;
            double dQin = 0.0;
            double dQout = 0.0;
            double dHdam = 0.0;
            double dVol = 0.0;

            for (long lP = 0; lP < lInputDataNum; lP++)
            {   // 入力する伝送データ数分繰り返します。
                csCellDt = csInputCellData[lP].GetInterpolatedCellD1();
                dQin += csInputCellData[lP].Data(csCellDt[0], 0);
            }

            damConstRateConstDisc(dDt, dQin, m_csRMInf.m_dQin, ref dQout, m_csRMInf.m_dQout,
                                    m_csRMInf.m_dQc, m_csRMInf.m_da, m_csRMInf.m_dQmax,
                                    m_csRMInf.m_dDamH1, m_csRMInf.m_dDamH2,
                                    ref dHdam, ref m_csRMInf.m_dHdam,
                                    m_csRMInf.m_ldamHV, m_csRMInf.m_ddamH, m_csRMInf.m_ddamV,
                                    m_csRMInf.m_bato, ref dVol, ref m_csRMInf.m_dQ_tds,
                                    ref m_csRMInf.m_bflg_tds);

            m_csRMInf.m_dQout = dQout;
            m_csRMInf.m_dHdam = dHdam;
            m_csRMInf.m_dStorage = dVol;
            m_csRMInf.m_dQin = dQin;

            return 0;
        }


        /// <summary><para>method outline:</para>
        /// <para>モデル演算結果を外部のエレメントに対して公開する</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>long lRtn = DataFusion( lOutputDataNum, ref csOutputCellData)</para>
        /// </example>
        /// <param name="lOutputDataNum">出力情報数</param>
        /// <param name="csOutputCellData">出力情報配列</param>
        /// <returns>=0:正常 -1:異常</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        protected override long DataFusion(long lOutputDataNum, ref McSendCellDataIF[] csOutputCellData)
        {
            long lP;
            HySCellData[] csSndCellData = null;

            for(int i = 0; i < this.m_csRMInf.m_lOutDataQNum; i++)
            {
                lP = this.m_csRMInf.m_lOutQIndx[i];
                csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
                csSndCellData[0].m_dData[0] = m_csRMInf.m_dQout;
            }
            for (int i = 0; i < this.m_csRMInf.m_lOutDataHNum; i++)
            {
                lP = this.m_csRMInf.m_lOutHIndx[i];
                csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
                csSndCellData[0].m_dData[0] = m_csRMInf.m_dHdam;
            }
            for (int i = 0; i < this.m_csRMInf.m_lOutDataVNum; i++)
            {
                lP = this.m_csRMInf.m_lOutVIndx[i];
                csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
                csSndCellData[0].m_dData[0] = m_csRMInf.m_dStorage;
            }

            return 0;
        }


        //====================
        // その他必要なメソッド
        //====================

        /// <summary><para>method outline:</para>
        /// <para>プロパティ情報を設定する</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>SetProperty(csCellMdlPropertyInfo)</para>
        /// </example>
        /// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
        {
            bool bRtn = false;

            // 使用しやすいようにキャストしておく
            m_csRMInf = (DamConstantRateDischargingCalInfo)m_csCalInfo;

            // プロパティ設定
            McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;
            if (csPrptyInfo != null)
            {
                // 演算ステップ時刻設定
                this.m_csDltTime = new HySTime(csPrptyInfo.GetStepTime());
                // 各種係数設定
                csPrptyInfo.GetInfo("m_dQc", ref m_csRMInf.m_dQc);
                csPrptyInfo.GetInfo("m_da", ref m_csRMInf.m_da);
                csPrptyInfo.GetInfo("m_dQmax", ref m_csRMInf.m_dQmax);
                csPrptyInfo.GetInfo("m_dDamH1", ref m_csRMInf.m_dDamH1);
                csPrptyInfo.GetInfo("m_bato", ref m_csRMInf.m_bato);
                csPrptyInfo.GetInfo("m_dDamH2", ref m_csRMInf.m_dDamH2);
                csPrptyInfo.GetInfo("m_sHvFile", ref m_csRMInf.m_sHvFile);

                bRtn = true;

                //相対パスを絶対パスに変換
                //m_csRMInf.m_sHvFile = CommonMP.HYMCO.OptionImpl.CommonControlLibrary.CommonLib.GetFullPath(m_csRMInf.m_sHvFile);

                //相対パス設定値を絶対パス化
                string sWorkFolder = m_csRMInf.m_sHvFile;
                sWorkFolder = sWorkFolder.Replace(".\\", "");
                sWorkFolder = this.GetProjectGroupDirectory().ToString() + "\\" + sWorkFolder;

                //H-Vファイルの存在をチェックする
                if (!System.IO.File.Exists(sWorkFolder))
                {
                    McLog.DebugOut(this.m_csSimTime, m_csElement.GetID(),
                                    "DamConstantRateDischargingModel", "SetProperty", "Cannot open data file(" + sWorkFolder + ")");
                    return false;
                }

                m_csRMInf.m_sHvFile = sWorkFolder;
            }

            m_csRMInf.m_ldamHV = 0;
            string buf = "";
            int i = 0;
            try
            {
                using (StreamReader sr = new StreamReader(
                   m_csRMInf.m_sHvFile, Encoding.GetEncoding("Shift_JIS")))
                {
                    while ((buf = sr.ReadLine()) != null)
                    {
                        string[] str = buf.Split(',');
                        if (m_csRMInf.m_ldamHV == 0)
                        {
                            m_csRMInf.m_ldamHV = long.Parse(str[1]);
                            m_csRMInf.m_ddamH = new double[m_csRMInf.m_ldamHV];
                            m_csRMInf.m_ddamV = new double[m_csRMInf.m_ldamHV];
                        }
                        else
                        {
                            if (char.IsDigit(str[0].Trim(), 0))
                            {
                                m_csRMInf.m_ddamH[i] = double.Parse(str[0]);
                                m_csRMInf.m_ddamV[i] = double.Parse(str[1]);
                                i += 1;
                            }
                        }
                    }

                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            return bRtn;
        }
        /// <summary><para>method outline:</para>
        /// <para>演算実行中断中のプロパティ等情報設定</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>bool bRtn = SetOnlineProperty(csCellMdlPropertyInfo)</para>
        /// </example>
        /// <param name="csCellMdlPropertyInfo">プロパティ情報</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks><para>remarks:</para>
        /// <para>演算中断中に　プロパティ情報を変更した場合にコールされる</para>
        /// </remarks>
        public override bool SetOnlineProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
        {
            bool bRtn = false;
            // プロパティ設定
            McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;
            if (csPrptyInfo != null)
            {
                bRtn = true;

            }
            return bRtn;
        }


        /// <summary>
        /// ダムの計算＜定率定量　貯水位考慮＞
        /// </summary>
        /// <param name="ddt">Δｔ</param>
        /// <param name="QR_U">流入量現時刻</param>
        /// <param name="QR_U_b">流入量前時刻</param>
        /// <param name="QR_D_sub">放流量現時刻</param>
        /// <param name="QR_D_b">放流量前時刻</param>
        /// <param name="damH3">定率定量パラメータ</param>
        /// <param name="damH4">定率定量パラメータ</param>
        /// <param name="damH5">定率定量パラメータ</param>
        /// <param name="damH1">定率定量パラメータ</param>
        /// <param name="damH2">定率定量パラメータ</param>
        /// <param name="Hdam_sub">貯水位現時刻</param>
        /// <param name="Hdam_b">前時刻貯水位</param>
        /// <param name="idamHV">H-Vの組数</param>
        /// <param name="ddamH">H-Vのデータ</param>
        /// <param name="ddamV">H-Vのデータ</param>
        /// <param name="dvol">後放流のフラグ</param>
        /// <param name="QR_D_tds">ただし書き放流量</param>
        /// <param name="flg_tds">ただし書きフラグ</param>
        public void damConstRateConstDisc(double ddt, double QR_U, double QR_U_b,
            ref double QR_D_sub, double QR_D_b, double damH3, double damH4,
            double damH5, double damH1, double damH2, ref double Hdam_sub,
            ref double Hdam_b, long ldamHV, double[] ddamH, double[] ddamV,
            bool bato, ref double dvol, ref double QR_D_tds, ref bool flg_tds)
        {
            double QR_D = QR_D_sub;
            double Hdam = Hdam_sub;
            double ddamVol_b = 0;
            double QOCAL = 0;
            double dq = 0;
            double ddamVol = 0; // ダムに貯水しているボリュームＶ(m3)


            // 前時刻貯水位から前時刻のダム貯水量Ｖを算出
            for (long l = 1; l < ldamHV; l++) //V←HVから内挿する
            {
                if (Hdam_b >= ddamH[l - 1] && Hdam_b < ddamH[l])
                {
                    ddamVol_b = ((Hdam_b - ddamH[l - 1]) * ddamV[l]
                        + (ddamH[l] - Hdam_b) * ddamV[l - 1])
                        / (ddamH[l] - ddamH[l - 1]);
                    break;
                }
            }

            // 放流量を予測
            if (QR_U <= damH3 && QR_D_b < damH3)     // 流入量 < カット開始流量
            {
                QR_D = QR_U;
                flg_tds = false;
            }
            else
            {
                QOCAL = damH4 * (QR_U - damH3) + damH3;
                if (QOCAL >= damH5)
                {
                    QOCAL = damH5; // 計算放流量>計画最大放流量 --> 放流量=計画最大放流量
                }
                if (QR_U <= QR_U_b)  // 流入量下降時
                {
                    if (QR_U > QR_D_b) // 流入量>前時刻の放流量の場合
                    {
                        QR_D = QR_D_b; // --> 放流量=前時刻放流量継続             
                    }
                    else               // 流入量<前時刻の放流量の場合
                    {
                        if (bato) // 後放流ありの場合
                        {
                            if (flg_tds)
                            {
                                if (QR_D_tds < QR_U)
                                {
                                    QR_D = QR_U;   // 放流量=流入量
                                }
                                else
                                {
                                    if (Hdam_b > damH2) // 制限水位に達するまで
                                    {
                                        QR_D = QR_D_tds; // 放流量=前時刻放流量継続
                                    }
                                    else                  // 制限水位に達すれば
                                    {
                                        QR_D = QR_U;   // 放流量=流入量
                                    }
                                }

                            }
                            else
                            {
                                if (Hdam_b > damH2) // 制限水位に達するまで
                                {
                                    QR_D = QR_D_b; // 放流量=前時刻放流量継続
                                }
                                else                  // 制限水位に達すれば
                                {
                                    QR_D = QR_U;   // 放流量=流入量
                                }
                            }
                        }
                        else // 後放流なしの場合
                        {
                            QR_D = QR_U; // 放流量=流入量
                        }
                    }
                }
                else  // 流入量上昇期
                {
                    //定率定量放流
                    if (QOCAL >= QR_D_b)
                    {
                        QR_D = QOCAL;
                    }
                    else
                    {
                        QR_D = QR_D_b;
                    }
                }
            }

            // 貯水位の算定
            dq = (QR_U - QR_D) * ddt; // δｔあたりダム貯留量Ｖ(m3)
            ddamVol = ddamVol_b + dq; // 現ダム貯水量(m3)
            if (ddamVol <= 0) ddamVol = 0.0;
            for (long l = 1; l < ldamHV; l++) // H←HVから内挿
            {
                if (ddamVol >= ddamV[l - 1] && ddamVol < ddamV[l])
                {
                    Hdam = ((ddamVol - ddamV[l - 1]) * ddamH[l] + (ddamV[l] - ddamVol)
                        * ddamH[l - 1]) / (ddamV[l] - ddamV[l - 1]);
                    break;
                }
            }

            // 洪水時満水位(damH1)を越えた場合の処理
            if (Hdam > damH1)
            {
                QR_D = QR_U; // Qin=Qout
                Hdam = Hdam_b;
                if (!flg_tds)
                {
                    QR_D_tds = QR_D_b;
                    flg_tds = true;
                }
            }
            // 出力変数の書換
            QR_D_sub = QR_D;
            Hdam_sub = Hdam;
            dvol = ddamVol;
        }
    }
}
