// <summary>ソースコード：ダム（一定量、定率定量方式）モデル用プロパティ画面クラス</summary>
// <author>CommonMP</author>

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


using CommonMP.HYSSOP.Interface.HSData;
using CommonMP.HYSSOP.Interface.HSController;
using CommonMP.HYSSOP.Interface.HSViewer;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSController;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface;
using CommonMP.HYMCO.Interface.Viewer;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Controller;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreOptions.McViewer.DotNetViewer;

// ToDo namespace は　モデル開発者が変更して、ユニークな名称にして下さい
namespace jp.or.jccajsce.DamConstantRateDischargingModel
{
    /// <summary><para>class outline:</para>
    /// <para>ダム（一定量、定率定量方式）モデル用プロパティ画面用 .net のフォーム</para>
    /// </summary>
    /// <remarks><para>history:</para>
    /// <para>[CommonMP][ver 1.0.0][2010/02][新規作成]</para>
    /// <para>remarks:</para>
    /// <para>必ず　McDotNetModelCellPropertyDetailBaseForm, McModelCellPropertyDetailFormIFから派生の事</para>
    /// </remarks>
   public partial class DamConstantRateDischargingModelDetailForm : McDotNetModelCellPropertyDetailBaseForm, McModelCellPropertyDetailFormIF
     {
        /// <summary> 演算間隔 </summary>
        private double m_dSec = 0.0;
        /// <summary> カット開始流量 </summary>
        private double m_dQc = 0.0;
        /// <summary> 放流率 </summary>
        private double m_da = 0.0;
        /// <summary> 計画最大放流量 </summary>
        private double m_dQmax = 0.0;
        /// <summary> ただし書き操作開始水位 </summary>
        private double m_dDamH1 = 0.0;
        /// <summary>後放流あり、なし</summary>
        private bool m_bato = false;
        /// <summary> 洪水期制限水位 </summary>
        private double m_dDamH2 = 0.0;
        /// <summary> H-Vテーブルファイル入力 </summary>
        private string m_sHvFile = "";
        /// <summary> 初期貯水位 </summary>
        private double m_dInitStage = 0.0;

        /// <summary>
        /// <para>method outline:</para>
        /// <para>デフォルトコンストラクタ</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para>McMyModelDetailForm csPropertyDetailForm = new McMyModelDetailForm();</para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks>
        /// <para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public DamConstantRateDischargingModelDetailForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// <para>method outline:</para>
        /// <para>プロパティ表示編集</para>
        /// </summary>
        /// <example>
        /// <para>usage:</para>
        /// <para>EditParameter( );</para>
        /// </example>
        /// <param name="">無し</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks>
        /// <para>remarks:</para>
        /// <para>無し</para>
        /// </remarks>
        public void EditParameter()
        {

            McDotNetModelCellPropertyScreenBase csScreen = this.GetScreen() as McDotNetModelCellPropertyScreenBase;
            jp.or.JccaJsce.DamConstantRateDischargingProperty.CommonLib.BasePath = csScreen.GetProjectGroupDirectory().ToString();

            // 画面表示情報編集
            McModelPropertyInfo csModelPropertyInfo = m_csPropertyInfo as McModelPropertyInfo;
            if (csModelPropertyInfo != null)
            {
                m_dSec = csModelPropertyInfo.GetStepTime();
                csModelPropertyInfo.GetInfo("m_dQc", ref m_dQc);
                csModelPropertyInfo.GetInfo("m_da", ref m_da);
                csModelPropertyInfo.GetInfo("m_dQmax", ref m_dQmax);
                csModelPropertyInfo.GetInfo("m_dDamH1", ref m_dDamH1);
                csModelPropertyInfo.GetInfo("m_bato", ref m_bato);
                csModelPropertyInfo.GetInfo("m_dDamH2", ref m_dDamH2);
                csModelPropertyInfo.GetInfo("m_sHvFile", ref m_sHvFile);
            }
            //初期値
            McInitialInfo csModelInitialInfo = m_csInitialInfo as McInitialInfo;
            if (csModelInitialInfo != null)
            {
                csModelInitialInfo.GetInfo("m_dInitStage", ref m_dInitStage);
            }
            txtSec.Text = m_dSec.ToString();
            txtQc.Text = m_dQc.ToString();
            txtA.Text = m_da.ToString();
            txtQmax.Text = m_dQmax.ToString();
            txtDamH1.Text = m_dDamH1.ToString();
            chkAto.Checked = m_bato;
            txtDamH2.Text = m_dDamH2.ToString();
            txtFName.Text = m_sHvFile;
            txtInitStage.Text = m_dInitStage.ToString();
        }

        /// <summary><para>method outline:</para>
        /// <para>設定処理</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>ParaSet_Click(sender,e);</para>
        /// </example>
        /// <param name="sender">object</param>
        /// <param name="e">Event</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks>
        /// <para>remarks:</para>
        /// <para>「設定」ボタンを押下された時の処理を記述</para>
        /// </remarks>
        private void ParaSet_Click(object sender, EventArgs e)
        {
            // 整合性チェック
            if (!isInputCheck())
                return;

            // 既存の情報と変更があるかチェック
            if (isDataCheck())
            {
                McModelPropertyInfo csModelPropertyInfo = m_csPropertyInfo as McModelPropertyInfo;
                csModelPropertyInfo.SetStepTime((double)txtSec.exValue);
                csModelPropertyInfo.SetInfo("m_dQc", (double)txtQc.exValue);
                csModelPropertyInfo.SetInfo("m_da", (double)txtA.exValue);
                csModelPropertyInfo.SetInfo("m_dQmax", (double)txtQmax.exValue);
                csModelPropertyInfo.SetInfo("m_dDamH1", (double)txtDamH1.exValue);
                csModelPropertyInfo.SetInfo("m_bato", chkAto.Checked);
                csModelPropertyInfo.SetInfo("m_dDamH2", (double)txtDamH2.exValue);
                csModelPropertyInfo.SetInfo("m_sHvFile", txtFName.Text);
                McInitialInfo csModelInitialInfo = m_csInitialInfo as McInitialInfo;
                csModelInitialInfo.SetInfo("m_dInitStage", (double)txtInitStage.exValue);
                DialogResult = DialogResult.OK;     // 設定変更がある場合
            }
            else
            {
                DialogResult = DialogResult.Cancel; // 設定変更がない場合
            }
            Close();
        }

        /// <summary><para>method outline:</para>
        /// <para>キャンセル処理</para>
        /// </summary>
        /// <example><para>usage:</para>
        /// <para>Cancel_Click(sender,e);</para>
        /// </example>
        /// <param name="sender">Object</param>
        /// <param name="e">Event</param>
        /// <returns>無し</returns>
        /// <exception cref="">無し</exception>
        /// <remarks>
        /// <para>remarks:</para>
        /// <para>「キャンセル」ボタンを押下された時の処理を記述</para>
        /// </remarks>
        private void Cancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        /// <summary>
        ///　データチェック処理
        /// </summary>
        /// <returns></returns>
        private bool isInputCheck()
        {
            if (txtSec.exIsEmpty)
            {
                ShowInputError(Properties.Resources.STRING_ERROR_MESSAGE1);
                txtSec.Focus();
                return false;
            }
            if (txtQc.exIsEmpty)
            {
                ShowInputError(Properties.Resources.STRING_ERROR_MESSAGE2);
                txtQc.Focus();
                return false;
            }
            if (txtA.exIsEmpty)
            {
                ShowInputError(Properties.Resources.STRING_ERROR_MESSAGE3);
                txtA.Focus();
                return false;
            }
            if (txtQmax.exIsEmpty)
            {
                ShowInputError(Properties.Resources.STRING_ERROR_MESSAGE4);
                txtQmax.Focus();
                return false;
            }
            if (txtDamH1.exIsEmpty)
            {
                ShowInputError(Properties.Resources.STRING_ERROR_MESSAGE5);
                txtDamH1.Focus();
                return false;
            }
            if (txtDamH2.exIsEmpty)
            {
                ShowInputError(Properties.Resources.STRING_ERROR_MESSAGE6);
                txtDamH2.Focus();
                return false;
            }
            if (txtFName.exIsEmpty)
            {
                ShowInputError(Properties.Resources.STRING_ERROR_MESSAGE7);
                txtFName.Focus();
                return false;
            }

            //ファイル存在チェック20140925
            if (!txtFName.exIsFileExists)
            {
                ShowInputError("'" + txtFName.Text + Properties.Resources.STRING_ERROR_MESSAGE8);
                txtFName.Focus();
                return false;
            }

            if (txtInitStage.exIsEmpty)
            {
                ShowInputError(Properties.Resources.STRING_ERROR_MESSAGE9);
                txtInitStage.Focus();
                return false;
            }

            return true;
        }

        /// <summary>
        /// 既存の情報と変更があるかチェック
        /// 変更があった場合、入力データの保存をする。
        /// </summary>
        /// <returns></returns>
        private bool isDataCheck()
        {
            //bool bflg = true;
            bool bflg = false;
           
            try
            {
                if (m_dSec != (double)txtSec.exValue)
                {
                    bflg = true;
                }
                if (m_dQc != (double)txtQc.exValue)
                {
                    bflg = true;
                }
                if (m_da != (double)txtA.exValue)
                {
                    bflg = true;
                }
                if (m_dQmax != (double)txtQmax.exValue)
                {
                    bflg = true;
                }
                if (m_dDamH1 != (double)txtDamH1.exValue)
                {
                    bflg = true;
                }
                if (m_bato != chkAto.Checked)
                {
                    bflg = true;
                }
                if (m_dDamH2 != double.Parse(txtDamH2.Text))
                {
                //修正
                    bflg = true;
                }
                if (m_sHvFile != txtFName.Text)
                {
                    bflg = true;
                }
                if (m_dInitStage != (double)txtInitStage.exValue)
                {
                    bflg = true;
                }
            }
            catch (Exception e)
            {
                ShowInputError(e.Message);
                // jp.or.JccaJsce.DamConstantRateDischargingProperty.CommonLib.ShowInputError(e.Message);
                bflg = false;
            }
            return bflg;
        }

        /// <summary>
        /// Ｈ－Ｖテーブルファイル入力を設定します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            McDotNetModelCellPropertyScreenBase csScreen = this.GetScreen() as McDotNetModelCellPropertyScreenBase;

            //OpenFileDialogクラスのインスタンスを作成
            OpenFileDialog ofd = new OpenFileDialog();

            //はじめのファイル名を指定する
            //はじめに「ファイル名」で表示される文字列を指定する
            ofd.FileName = "";

            //はじめに表示されるフォルダを指定する
            ofd.InitialDirectory = csScreen.GetProjectGroupDirectory().ToString();//HySEnvInf.GetHomeDirectory().ToString();

            //[ファイルの種類]に表示される選択肢を指定する
            ofd.Filter = Properties.Resources.STRING_ERROR_MESSAGE10;

            //[ファイルの種類]ではじめに
            ofd.FilterIndex = 1;

            //タイトルを設定する
            ofd.Title = Properties.Resources.STRING_ERROR_MESSAGE11;
            
            //カレントフォルダを入力値に設定
            

            string sPath = txtFName.exValidDirectory;            
            string sName = txtFName.exFileNameRemovePath;
            if (sName.Length > 0)
            {
                ofd.FileName = sName;
            }
            if (sPath.Length > 0)
            {
                ofd.InitialDirectory = sPath;
            }

            //ダイアログを表示する
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                ////OKボタンがクリックされたとき
                ////選択されたファイル名を表示する
                //txtFName.Text = ofd.FileName;
                
                // 相対パス化（プロジェクトグループ名からの相対化）
                HySString stmp1, stmp2 = new HySString("");;
                stmp1 = new HySString(ofd.FileName);
                csScreen.GetRelativePathBasedOnProjectGroup(stmp1, ref stmp2);
                txtFName.Text = ".\\" + stmp2.ToString();
            }
  
        }

        /// <summary>
        /// 後放流あり・なし
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkAto_CheckedChanged(object sender, EventArgs e)
        {
            if (chkAto.Checked)
                chkAto.Text = Properties.Resources.STRING_ERROR_MESSAGE12;
            else
                chkAto.Text = Properties.Resources.STRING_ERROR_MESSAGE13;
        }


        /// <summary>
        /// 入力エラーメッセージを表示する
        /// </summary>
        /// <param name="sMessage">エラーメッセージ</param>
        public static void ShowInputError(string sMessage)
        {
            MessageBox.Show(sMessage, "err", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }
    }
}
